'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };
// *


exports.default = transformCssModules;

var _path = require('path');

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _fs = require('fs');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* /
const writeFileSync = (file, content) => {
    console.log(`Will save ${file}\n${content.replace(/^/gm, '  ')}`);
};
// */

var writeCssFile = function writeCssFile(filename, content) {
    _mkdirp2.default.sync((0, _path.dirname)(filename));
    (0, _fs.writeFileSync)(filename, content);
};
var appendCssFile = function appendCssFile(filename, content) {
    _mkdirp2.default.sync((0, _path.dirname)(filename));
    (0, _fs.appendFileSync)(filename, content);
};

var simpleRequires = ['createImportedName', 'generateScopedName', 'processCss', 'preprocessCss'];

var complexRequires = ['append', 'prepend'];

var defaultOptions = {
    generateScopedName: '[name]__[local]___[hash:base64:5]'
};

function transformCssModules(_ref) {
    var t = _ref.types;

    function resolveModulePath(filename) {
        var dir = (0, _path.dirname)(filename);
        if ((0, _path.isAbsolute)(dir)) return dir;
        if (process.env.PWD) return (0, _path.resolve)(process.env.PWD, dir);
        return (0, _path.resolve)(dir);
    }

    /**
     *
     * @param {String} filepath     javascript file path
     * @param {String} cssFile      requireed css file path
     * @returns {Array} array of class names
     */
    function requireCssFile(filepath, cssFile) {
        var filePathOrModuleName = cssFile;

        // only resolve path to file when we have a file path
        if (!/^\w/i.test(filePathOrModuleName)) {
            var from = resolveModulePath(filepath);
            filePathOrModuleName = (0, _path.resolve)(from, filePathOrModuleName);
        }
        return require(filePathOrModuleName);
    }

    // is css modules require hook initialized?
    var initialized = false;

    var matchExtensions = /\.css$/i;
    function matcher() {
        var extensions = arguments.length <= 0 || arguments[0] === undefined ? ['.css'] : arguments[0];

        var extensionsPatern = extensions.join('|').replace(/\./g, '\\\.');
        return new RegExp('(' + extensionsPatern + ')$', 'i');
    }

    return {
        visitor: {
            Program: function Program(path, state) {
                if (initialized) {
                    return;
                }

                var currentConfig = _extends({}, defaultOptions, state.opts);
                // this is not a css-require-ook config
                delete currentConfig.extractCss;

                // match file extensions, speeds up transform by creating one
                // RegExp ahead of execution time
                matchExtensions = matcher(currentConfig.extensions);

                // Add a space in current state for css filenames
                state.$$css = {
                    styles: new Map()
                };

                var extractCssFile = function extractCssFile(filepath, css) {
                    var _state$opts$extractCs = state.opts.extractCss;
                    var extractCss = _state$opts$extractCs === undefined ? null : _state$opts$extractCs;

                    if (!extractCss) return null;

                    // this is the case where a single extractCss is requested
                    if (typeof extractCss === 'string') {
                        // If this is the first file, then we should replace
                        // old content
                        if (state.$$css.styles.size === 1) {
                            return writeCssFile(extractCss, css);
                        }
                        // this should output in a single file.
                        // Let's append the new file content.
                        return appendCssFile(extractCss, css);
                    }

                    // This is the case where each css file is written in
                    // its own file.
                    var _extractCss$dir = extractCss.dir;
                    var dir = _extractCss$dir === undefined ? 'dist' : _extractCss$dir;
                    var _extractCss$filename = extractCss.filename;
                    var filename = _extractCss$filename === undefined ? '[name].css' : _extractCss$filename;
                    var _extractCss$relativeR = extractCss.relativeRoot;
                    var relativeRoot = _extractCss$relativeR === undefined ? '' : _extractCss$relativeR;

                    // Make css file narmpe relative to relativeRoot

                    var relativePath = (0, _path.relative)((0, _path.resolve)(process.cwd(), relativeRoot), filepath);
                    var destination = (0, _path.join)((0, _path.resolve)(process.cwd(), dir), filename).replace(/\[name]/, (0, _path.basename)(filepath, (0, _path.extname)(filepath))).replace(/\[path]/, relativePath);

                    writeCssFile(destination, css);
                };

                var pushStylesCreator = function pushStylesCreator(toWrap) {
                    return function (css, filepath) {
                        var processed = void 0;
                        if (typeof toWrap === 'function') {
                            processed = toWrap(css, filepath);
                        }
                        if (typeof processed !== 'string') processed = css;

                        if (!state.$$css.styles.has(filepath)) {
                            state.$$css.styles.set(filepath, processed);
                            extractCssFile(filepath, processed);
                        }

                        return processed;
                    };
                };

                // check if there are simple requires and if they are functions
                simpleRequires.forEach(function (key) {
                    if (typeof currentConfig[key] !== 'string') {
                        return;
                    }

                    var modulePath = (0, _path.resolve)(process.cwd(), currentConfig[key]);

                    // this one can be require or string
                    if (key === 'generateScopedName') {
                        try {
                            // if it is existing file, require it, otherwise use value
                            currentConfig[key] = require(modulePath);
                        } catch (e) {
                            try {
                                currentConfig[key] = require(currentConfig[key]);
                            } catch (_e) {
                                // do nothing, because it is not a valid path
                            }
                        }

                        if (typeof currentConfig[key] !== 'function' && typeof currentConfig[key] !== 'string') {
                            throw new Error('Configuration \'' + key + '\' is not a string or function.');
                        }

                        return;
                    }

                    if (currentConfig.hasOwnProperty(key)) {
                        try {
                            currentConfig[key] = require(modulePath);
                        } catch (e) {
                            try {
                                currentConfig[key] = require(currentConfig[key]);
                            } catch (_e) {
                                // do nothing because it is not a valid path
                            }
                        }

                        if (typeof currentConfig[key] !== 'function') {
                            throw new Error('Module \'' + modulePath + '\' does not exist or is not a function.');
                        }
                    }
                });

                // wrap or define processCss function that collect generated css
                currentConfig.processCss = pushStylesCreator(currentConfig.processCss);

                complexRequires.forEach(function (key) {
                    if (!currentConfig.hasOwnProperty(key)) {
                        return;
                    }

                    if (!Array.isArray(currentConfig[key])) {
                        throw new Error('Configuration \'' + key + '\' has to be an array.');
                    }

                    currentConfig[key].forEach(function (plugin, index) {
                        // first try to load it using npm
                        try {
                            currentConfig[key][index] = require(plugin);
                        } catch (e) {
                            try {
                                currentConfig[key][index] = require((0, _path.resolve)(process.cwd(), plugin));
                            } catch (_e) {
                                // do nothing
                            }
                        }

                        if (typeof currentConfig[key][index] !== 'function') {
                            throw new Error('Configuration \'' + key + '\' has to be valid path to a module at index ' + index + ' or it does not export a function.');
                        }

                        currentConfig[key][index] = currentConfig[key][index]();
                    });
                });

                require('css-modules-require-hook')(currentConfig);

                initialized = true;
            },
            ImportDeclaration: function ImportDeclaration(path, _ref2) {
                var file = _ref2.file;

                // this method is called between enter and exit, so we can map css to our state
                // it is then replaced with require call which will be handled in seconds pass by CallExpression
                // CallExpression will then replace it or remove depending on parent node (if is Program or not)
                var value = path.node.source.value;


                if (matchExtensions.test(value)) {
                    var requiringFile = file.opts.filename;
                    requireCssFile(requiringFile, value);
                }
            },
            CallExpression: function CallExpression(path, _ref3) {
                var file = _ref3.file;
                var _path$node = path.node;
                var calleeName = _path$node.callee.name;
                var args = _path$node.arguments;


                if (calleeName !== 'require' || !args.length || !t.isStringLiteral(args[0])) {
                    return;
                }

                var _args = _slicedToArray(args, 1);

                var stylesheetPath = _args[0].value;


                if (matchExtensions.test(stylesheetPath)) {
                    (function () {
                        var requiringFile = file.opts.filename;
                        var tokens = requireCssFile(requiringFile, stylesheetPath);

                        // if parent expression is not a Program, replace expression with tokens
                        // Otherwise remove require from file, we just want to get generated css for our output
                        if (!t.isExpressionStatement(path.parent)) {
                            /* eslint-disable new-cap */
                            path.replaceWith(t.ObjectExpression(Object.keys(tokens).map(function (token) {
                                return t.ObjectProperty(t.StringLiteral(token), t.StringLiteral(tokens[token]));
                            })));
                        } else {
                            path.remove();
                        }
                    })();
                }
            }
        }
    };
}